from pymongo.database import Database
from typing import TypedDict, List, Optional
import datetime as dt


class CBOEDocument(TypedDict):
    option: str
    bid: float
    bid_size: float
    ask: float
    ask_size: float
    iv: float
    open_interest: float
    volume: float
    delta: float
    gamma: float
    theta: float
    rho: float
    vega: float
    theo: float
    change: float
    open: float
    high: float
    low: float
    tick: str
    last_trade_price: float
    last_trade_time: dt.datetime
    percent_change: float
    prev_day_close: float


class CBOEQueries:
    def __init__(self, db: Database, collection_name: str):
        self.collection_name = collection_name
        self.collection = db["radar-cboe"]

    def insert_many(
        self, documents: List[CBOEDocument], now: Optional[dt.datetime] = None
    ):
        if now is None:
            now = dt.datetime.now(dt.timezone.utc)
        row_date = dt.datetime(
            year=now.year,
            month=now.month,
            day=now.day,
            hour=now.hour,
            minute=(now.minute // 5) * 5,
            second=0,
        )
        for i in range(len(documents)):
            documents[i]["row_date"] = row_date
        return self.collection.insert_many(documents)



from pymongo.database import Database
from typing import TypedDict, Optional
import datetime as dt
from enum import Enum


class OptionType(str, Enum):
    P: str = "P"
    C: str = "C"


class OpenInterestDocument(TypedDict):
    open_interest: float
    option_type: OptionType
    strike: str


class OpenInterestQueries:
    def __init__(self, db: Database, collection_name: str):
        self.collection_name = collection_name
        self.open_interest_collection = db["radar-open-interest"]
        self.cboe_collection = db["radar-cboe"]

    def insert(self, now: Optional[dt.datetime] = None):
        if now is None:
            now = dt.datetime.now(dt.timezone.utc)
        now = dt.datetime(
            year=now.year,
            month=now.month,
            day=now.day,
            hour=now.hour,
            minute=now.minute,
            second=0,
        )
        result = self.cboe_collection.aggregate(
            [
                {
                    "$match": {"row_date": now - dt.timedelta(seconds=15 * 60)},
                },
                {
                    "$project": {
                        "option_type": {"$substr": ["$option", 9, 1]},
                        "strike": {"$substr": ["$option", 11, 4]},
                        "open_interest": True,
                    }
                },
                {"$match": {"option_type": {"$in": ["C", "P"]}}},
                {
                    "$group": {
                        "_id": {"option_type": "$option_type", "strike": "$strike"},
                        "open_interest": {"$sum": "$open_interest"},
                    }
                },
                {
                    "$project": {
                        "_id": False,
                        "option_type": "$_id.option_type",
                        "strike": "$_id.strike",
                        "open_interest": True,
                    }
                },
                {
                    "$sort": {
                        "strike": 1,
                        "option_type": -1,
                    }
                },
            ]
        )
        documents = list(result)
        for i in range(len(documents)):
            documents[i]["datetime"] = dt.datetime(
                year=now.year, month=now.month, day=now.day, hour=0, minute=0, second=0
            )
        self.open_interest_collection.insert_many(documents)



from pymongo.database import Database

from .collections.cboe import CBOEQueries
from .collections.spot import SpotQueries
from .collections.open_interest import OpenInterestQueries


class QueryManager:
    def __init__(
        self,
        db: Database,
        cboe_collection_name: str,
        spot_collection_name: str,
        open_interest_collection_name: str,
    ):
        self.db = db
        self.cboe_queries = CBOEQueries(db=db, collection_name=cboe_collection_name)
        self.spot_queries = SpotQueries(db=db, collection_name=spot_collection_name)
        self.open_interest = OpenInterestQueries(
            db=db, collection_name=open_interest_collection_name
        )



from pymongo.database import Database
from typing import TypedDict
import datetime as dt


class SpotDocument(TypedDict):
    current_price: float


class SpotQueries:
    def __init__(self, db: Database, collection_name: str):
        self.collection_name = collection_name
        self.collection = db["radar-spot"]

    def insert_many(self, document: SpotDocument, now: dt.datetime):
        row_date = dt.datetime(
            year=now.year,
            month=now.month,
            day=now.day,
            hour=now.hour,
            minute=now.minute,
            second=0,
        )
        document["row_date"] = row_date
        return self.collection.insert_one(document)
